window.Gleam.initSearch([{"doc":"formz","title":"formz","content":"# formz\n\n[![Package Version](https://img.shields.io/hexpm/v/formz)](https://hex.pm/packages/formz)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/formz/)\n\nA Gleam library for parsing and generating accessible HTML forms.\n\nHTML forms rendered in the browser and the data they are parsed into are\nintrinsically linked. Treating the markup and the parsing as two separate\nproblems to solve is inconvenient and leads to bugs. This library aims\nto make that link explicit and easy to manage, while making it really easy\nto make accessible forms.\n\n```sh\ngleam add formz@0.1\n```\n\n## Creating a form\n\nA `formz` form is a list of fields and a decoder function.\n\n&gt; **Note:** This library currently has two non-interoperable ways to define forms,\none using the builder pattern, and one using a series of `use` calls like with\nthe [toy](https://hexdocs.pm/toy/) or [decode/zero](https://hexdocs.pm/decode/)\npackages.  After gathering some feedback, only one of them will be kept and this\nlibrary will be moved do version 1.0.0.\n\n### builder pattern\n\nWith the builder pattern, you add the fields and then explicitly specify the\ndecoder function...\n\n```gleam\nimport formz/field.{field}\nimport formz/formz_builder as formz\nimport formz_string/definitions\n\npub fn make_form() {\n  formz.new()\n  |&gt; formz.require(field(&quot;username&quot;), definitions.text_field())\n  |&gt; formz.require(field(&quot;password&quot;), definitions.password_field())\n  |&gt; formz.decodes(fn(username) { fn(password) { #(username, password) } })\n}\n```\n\n### `use`/callbacks pattern\n\nWith the `use`/callbakcks pattern, you create the decoder function as you add\nthe fields...\n\n```gleam\nimport formz/field.{field}\nimport formz/formz_use as formz\nimport formz_string/definitions\n\npub fn make_form() {\n  use username &lt;- formz.require(field(&quot;username&quot;), definitions.text_field())\n  use password &lt;- formz.require(field(&quot;password&quot;), definitions.password_field())\n\n  formz.create_form(#(username, password))\n}\n```\n\n## Creating fields\n\nThere are two arguments to adding a field to a form (seen above):\n\n1. A [Field](https://hexdocs.pm/formz/formz/field.html), which holds specific,\n   unique details about the field, such as its name, label, help text, disabled\n   state, etc.\n2. A [Definition](https://hexdocs.pm/formz/formz/definition.html), which\n   says (A) how to generate the HTML input element for the field, and (B) how\n   to parse the data from the field. These definitions are reusable and can be\n   shared across fields, forms and projects.\n\n### Field details\n\n```gleam\n// name is required, the other details are optional\nfield(named: &quot;username&quot;)\n|&gt; field.set_label(&quot;Username&quot;)\n|&gt; field.set_help_text(&quot;Only alphanumeric characters are allowed.&quot;)\n```\n\n```gleam\nfield(named: &quot;userid&quot;) |&gt; field.make_hidden |&gt; field.set_raw_value(&quot;42&quot;)\n```\n\n### Field definition\n\n[Defintions](https://hexdocs.pm/formz/formz/definition.html) are the heavy\ncompared to the lightness of fields; they take a bit more work to make as they\nare intended to be more reusable.\n\nThe first role of a `Defintion` is to generate the HTML widget for the field.\nThis library is format-agnostic and you can generate HTML widgets as raw\nstrings, Lustre elements, Nakai nodes, something else, etc. There are\ncurrently three `formz` libraries that provide common field definitions for the\nmost common HTML formats.\n\n- [formz_string](https://hexdocs.pm/formz_string/)\n- [formz_nakai](https://hexdocs.pm/formz_nakai/)\n- [formz_lustre](https://hexdocs.pm/formz_lustre/) (untested in a browser,\n  would it be useful there??)\n\nThe second role  of a `Definition` is to parse the data from the field. There\nare a two parts to this, as how you parse a field&#39;s value depends on if it is\noptional or required.  Not all scenarios can be cookie-cutter placed into an\n`Option`. So you need to provide two parse functions, one for when a field is\nrequired, and a second for when it&#39;s optional.\n\n```gleam\n/// you won&#39;t often need to do this directly (I think??).  The idea is that\n/// there&#39;d be libs with the definitions you need.\n\nimport formz/definition.{Definition}\nimport formz/field\nimport formz/validation\nimport formz/widget\nimport lustre/attribute\nimport lustre/element\nimport lustre/element/html\n\nfn password_widget(\n  field: field.Field,\n  args: widget.Args,\n) -&gt; element.Element(msg) {\n  html.input([\n    attribute.type_(&quot;password&quot;),\n    attribute.name(field.name),\n    attribute.id(args.id),\n    attribute.attribute(&quot;aria-labelledby&quot;, field.label),\n  ])\n}\n\npub fn password_field() {\n  Definition(\n    widget: password_widget,\n    parse: validation.string,\n    optional_parse: fn(parse, str) {\n      case str {\n        &quot;&quot; -&gt; Ok(option.None)\n        _ -&gt; parse(str)\n      }\n    },\n    // We need to have a stub value for each parser. The stubs are used when\n    // building the decoder and parse functions for the form.\n    stub: &quot;&quot;,\n    optional_stub: option.None,\n  )\n}\n```\n\n\n\n## Generating HTML for a form\n\nGenerally speaking, the idea with a `formz` form is that you are not going\nto generate the HTML for each field individually, but rather, you&#39;d use\na function to loop through each field, generating semantic, accessible\nmarkup for each one.\n\nThe specifics of how you would do this are going\nto vary greatly for each project and its styling/markup needs.\n\nHowever, the three `formz_*` libraries mentioned above all provide a\nsimple form generator function that you can use as is, or as a starting\npoint for your own.  `formz` is BYOS, Bring Your Own Stylesheet, so the\nbuilt-in form generators come unstyled. If there is interest, I could add\na super simple CSS file to get the ball rolling and make the default\nforms easier to use out of the box.\n\nThat said, you can also create the form HTML yourself, directly for each field.\nThere&#39;s [an example](https://github.com/bentomas/formz/blob/main/formz_demo/src/formz_demo/examples/custom_output.gleam)\nin the demo project showing how to do this.\n\n### Generating form HTML using the `formz_string` library\n\nThe built-in form generators leave it as homework to add the form tags and\nsubmit buttons.\n\n```gleam\nimport formz_string/simple\n\npub fn show_form(form) -&gt; String {\n  &quot;&lt;form method=\\&quot;post\\&quot;&gt;&quot;\n  &lt;&gt; simple.generate_form(form)\n  &lt;&gt; &quot;&lt;p&gt;&lt;button type\\&quot;submit\\&quot;&gt;Submit&lt;/button&gt;&lt;/p&gt;&quot;\n  &lt;&gt; &quot;&lt;/form&gt;&quot;\n}\n```\n\n\n## Parsing form data\n\nYou can parse a `formz` form with a tuple of values and names, typically from\na POST request.  Here we parse in a `wisp` handler:\n\n```gleam\npub fn handle_form_submission(req: Request) -&gt; Response {\n  use formdata &lt;- wisp.require_form(req)\n\n  let result = make_form()\n  |&gt; formz.data(formdata.values)\n  |&gt; formz.parse\n\n  case result {\n    Ok(credentials) -&gt; {\n      let #(username, password) = credentials\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(&quot;Hello &quot;&lt;&gt;username&lt;&gt;&quot;!&quot;))\n    }\n    Error(form_with_errors) -&gt; {\n      show_form(form_with_errors)\n    }\n  }\n}\n```\n\nHowever, often you want to parse a form, and then... you know... act on that\ndata, and in doing so you might discover more errors for the form.  In this\nsituation you can use `parse_then_try`:\n\n```gleam\npub fn handle_form_submission(req: Request) -&gt; Response {\n  use formdata &lt;- wisp.require_form(req)\n\n  let result = make_form()\n  |&gt; formz.data(formdata.values)\n  |&gt; formz.parse_then_try(fn(form, credentials) {\n    case credentials {\n      #(&quot;admin&quot; as username, &quot;l33t&quot;) -&gt; Ok(username)\n      #(&quot;admin&quot;, _) -&gt;\n        form\n        |&gt; formz.update_field(&quot;password&quot;, field.set_error(_, &quot;Wrong password&quot;))\n        |&gt; Error\n      _ -&gt;\n        form\n        |&gt; formz.update_field(&quot;username&quot;, field.set_error(_, &quot;Unknown username&quot;))\n        |&gt; Error\n    }\n  })\n\n  case result {\n    Ok(username) -&gt; {\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(&quot;Hello &quot; &lt;&gt; username &lt;&gt; &quot;!&quot;))\n    }\n    Error(form_with_errors) -&gt; {\n      show_form(form_with_errors)\n    }\n  }\n}\n```\n\n## See it in action\n\nThere is a [demo wisp app](https://github.com/bentomas/formz/tree/main/formz_demo)\nshowing a few interactive examples of how `formz` works in the repo.\n","url":"index.html"},{"doc":"formz/field","title":"Field","content":"pub type Field {\n  Valid(\n    name: String,\n    label: String,\n    help_text: String,\n    disabled: Bool,\n    required: Bool,\n    hidden: Bool,\n    value: String,\n  )\n  Invalid(\n    name: String,\n    label: String,\n    help_text: String,\n    disabled: Bool,\n    required: Bool,\n    hidden: Bool,\n    value: String,\n    error: String,\n  )\n}\n\nValid(\n  name: String,\n  label: String,\n  help_text: String,\n  disabled: Bool,\n  required: Bool,\n  hidden: Bool,\n  value: String,\n)\n\n\nInvalid(\n  name: String,\n  label: String,\n  help_text: String,\n  disabled: Bool,\n  required: Bool,\n  hidden: Bool,\n  value: String,\n  error: String,\n)\n\n\nSynonyms:\nformz/field.Field\nformz/field Field","url":"formz/field.html#Field"},{"doc":"formz/field","title":"field","content":"pub fn field(named name: String) -&gt; Field\n\nSynonyms:\nformz/field.field\nformz/field field","url":"formz/field.html#field"},{"doc":"formz/field","title":"make_disabled","content":"pub fn make_disabled(field: Field) -&gt; Field\n\nSynonyms:\nformz/field.make_disabled\nformz/field make_disabled","url":"formz/field.html#make_disabled"},{"doc":"formz/field","title":"make_hidden","content":"pub fn make_hidden(field: Field) -&gt; Field\n\nSynonyms:\nformz/field.make_hidden\nformz/field make_hidden","url":"formz/field.html#make_hidden"},{"doc":"formz/field","title":"set_disabled","content":"pub fn set_disabled(field: Field, disabled: Bool) -&gt; Field\n\nSynonyms:\nformz/field.set_disabled\nformz/field set_disabled","url":"formz/field.html#set_disabled"},{"doc":"formz/field","title":"set_error","content":"pub fn set_error(field: Field, error: String) -&gt; Field\n\nSynonyms:\nformz/field.set_error\nformz/field set_error","url":"formz/field.html#set_error"},{"doc":"formz/field","title":"set_help_text","content":"pub fn set_help_text(field: Field, help_text: String) -&gt; Field\n\nSynonyms:\nformz/field.set_help_text\nformz/field set_help_text","url":"formz/field.html#set_help_text"},{"doc":"formz/field","title":"set_hidden","content":"pub fn set_hidden(field: Field, hidden: Bool) -&gt; Field\n\nSynonyms:\nformz/field.set_hidden\nformz/field set_hidden","url":"formz/field.html#set_hidden"},{"doc":"formz/field","title":"set_label","content":"pub fn set_label(field: Field, label: String) -&gt; Field\n\nSynonyms:\nformz/field.set_label\nformz/field set_label","url":"formz/field.html#set_label"},{"doc":"formz/field","title":"set_name","content":"pub fn set_name(field: Field, name: String) -&gt; Field\n\nSynonyms:\nformz/field.set_name\nformz/field set_name","url":"formz/field.html#set_name"},{"doc":"formz/field","title":"set_raw_value","content":"pub fn set_raw_value(field: Field, value: String) -&gt; Field\n\nSynonyms:\nformz/field.set_raw_value\nformz/field set_raw_value","url":"formz/field.html#set_raw_value"},{"doc":"formz/field","title":"formz/field","content":" A `Field` is the first argument needed to add a field to a form. It contains\n information about this specific field, like it&#39;s name, label, or (optional)\n help_text.","url":"formz/field.html"},{"doc":"formz/subform","title":"SubForm","content":"pub type SubForm {\n  SubForm(name: String, label: String, help_text: String)\n}\n\nSubForm(name: String, label: String, help_text: String)\n\n\nSynonyms:\nformz/subform.SubForm\nformz/subform SubForm","url":"formz/subform.html#SubForm"},{"doc":"formz/subform","title":"set_help_text","content":"pub fn set_help_text(sub: SubForm, help_text: String) -&gt; SubForm\n\nSynonyms:\nformz/subform.set_help_text\nformz/subform set_help_text","url":"formz/subform.html#set_help_text"},{"doc":"formz/subform","title":"set_label","content":"pub fn set_label(sub: SubForm, label: String) -&gt; SubForm\n\nSynonyms:\nformz/subform.set_label\nformz/subform set_label","url":"formz/subform.html#set_label"},{"doc":"formz/subform","title":"set_name","content":"pub fn set_name(sub: SubForm, name: String) -&gt; SubForm\n\nSynonyms:\nformz/subform.set_name\nformz/subform set_name","url":"formz/subform.html#set_name"},{"doc":"formz/subform","title":"subform","content":"pub fn subform(name: String) -&gt; SubForm\n\nSynonyms:\nformz/subform.subform\nformz/subform subform","url":"formz/subform.html#subform"},{"doc":"formz/subform","title":"formz/subform","content":"","url":"formz/subform.html"},{"doc":"formz/widget","title":"Args","content":"pub type Args {\n  Args(\n    id: String,\n    labelled_by: LabelledBy,\n    described_by: DescribedBy,\n  )\n}\n\nArgs(\n  id: String,\n  labelled_by: LabelledBy,\n  described_by: DescribedBy,\n)\n\n\nSynonyms:\nformz/widget.Args\nformz/widget Args","url":"formz/widget.html#Args"},{"doc":"formz/widget","title":"DescribedBy","content":"pub type DescribedBy {\n  DescribedByElementsWithIds(ids: List(String))\n  DescribedByNone\n}\n\nDescribedByElementsWithIds(ids: List(String))\n\n\nDescribedByNone\n\n\nSynonyms:\nformz/widget.DescribedBy\nformz/widget DescribedBy","url":"formz/widget.html#DescribedBy"},{"doc":"formz/widget","title":"LabelledBy","content":"pub type LabelledBy {\n  LabelledByLabelFor\n  LabelledByFieldValue\n  LabelledByElementsWithIds(ids: List(String))\n}\n\nLabelledByLabelFor\n\n\nLabelledByFieldValue\n\n\nLabelledByElementsWithIds(ids: List(String))\n\n\nSynonyms:\nformz/widget.LabelledBy\nformz/widget LabelledBy","url":"formz/widget.html#LabelledBy"},{"doc":"formz/widget","title":"Widget","content":"pub type Widget(format) =\n  fn(field.Field, Args) -&gt; format\n\n\nSynonyms:\nformz/widget.Widget\nformz/widget Widget","url":"formz/widget.html#Widget"},{"doc":"formz/widget","title":"args","content":"pub fn args(labelled_by labelled_by: LabelledBy) -&gt; Args\n\nSynonyms:\nformz/widget.args\nformz/widget args","url":"formz/widget.html#args"},{"doc":"formz/widget","title":"described_by","content":"pub fn described_by(args: Args, db: DescribedBy) -&gt; Args\n\nSynonyms:\nformz/widget.described_by\nformz/widget described_by","url":"formz/widget.html#described_by"},{"doc":"formz/widget","title":"id","content":"pub fn id(args: Args, str: String) -&gt; Args\n\nSynonyms:\nformz/widget.id\nformz/widget id","url":"formz/widget.html#id"},{"doc":"formz/widget","title":"formz/widget","content":"","url":"formz/widget.html"},{"doc":"formz","title":"FormItem","content":"pub type FormItem(format) {\n  Field(field.Field, widget: widget.Widget(format))\n  SubForm(subform.SubForm, items: List(FormItem(format)))\n}\n\nField(field.Field, widget: widget.Widget(format))\n\n\nSubForm(subform.SubForm, items: List(FormItem(format)))\n\n\nSynonyms:\nformz.FormItem\nformz FormItem","url":"formz.html#FormItem"},{"doc":"formz","title":"formz","content":" This will eventually be the home of `formz_builder` or `formz_use`.","url":"formz.html"},{"doc":"formz/definition","title":"Definition","content":"pub type Definition(format, required, optional) {\n  Definition(\n    widget: widget.Widget(format),\n    parse: fn(String) -&gt; Result(required, String),\n    stub: required,\n    optional_parse: fn(\n      fn(String) -&gt; Result(required, String),\n      String,\n    ) -&gt;\n      Result(optional, String),\n    optional_stub: optional,\n  )\n}\n\nDefinition(\n  widget: widget.Widget(format),\n  parse: fn(String) -&gt; Result(required, String),\n  stub: required,\n  optional_parse: fn(\n    fn(String) -&gt; Result(required, String),\n    String,\n  ) -&gt;\n    Result(optional, String),\n  optional_stub: optional,\n)\n\nwidget\n&lt;p&gt;The widget generates the HTML for the field.&lt;/p&gt;\n\nparse\n&lt;p&gt;This parse function takes the raw string from the parsed POST data\nand converts it to a Gleam type.  This &lt;code&gt;parse&lt;/code&gt; is for when a value\nis required, so it should return an error if the field is empty.&lt;/p&gt;\n\nstub\n&lt;p&gt;The &lt;code&gt;use&lt;/code&gt;/callbacks pattern for generating a form requires a stub\nvalue for each field, because the actual decode function is called\nstep by step as the fields are added to the form and &lt;code&gt;formz&lt;/code&gt; learns\nthe form’s details as it goes.  This stub value is purely used\nfor navigating the decode function, and just needs to match the type\nof the real value that can be parsed.&lt;/p&gt;\n\noptional_parse\n&lt;p&gt;If a field is marked as optional, this function is called, with the\nabove parse as an argument.  The idea is that this function will\ncall out to the parse function if the field is not empty, and\nthis should only handle the case where the raw input value is empty.\nThis function is necessary because not all fields should just be parsed\ninto an &lt;code&gt;Option&lt;/code&gt; when they aren’t provided.\nFor example, an optional text field might be an empty string,\nan optional checkbox might be &lt;code&gt;False&lt;/code&gt;, and an optional select might\nbe &lt;code&gt;option.None&lt;/code&gt;.&lt;/p&gt;\n\noptional_stub\n&lt;p&gt;stub for the optional_parse return value&lt;/p&gt;\n\nSynonyms:\nformz/definition.Definition\nformz/definition Definition","url":"formz/definition.html#Definition"},{"doc":"formz/definition","title":"make_simple_optional_parse","content":"pub fn make_simple_optional_parse() -&gt; fn(\n  fn(String) -&gt; Result(a, String),\n  String,\n) -&gt; Result(Option(a), String)\n A convenience function to make the simple optional parse function where\n if a value isn&#39;t provided, just return `option.None`, otherwise call out\n to the parse function and put it&#39;s value in `option.Some`.\nSynonyms:\nformz/definition.make_simple_optional_parse\nformz/definition make_simple_optional_parse","url":"formz/definition.html#make_simple_optional_parse"},{"doc":"formz/definition","title":"set_widget","content":"pub fn set_widget(\n  definition: Definition(a, b, c),\n  widget: fn(Field, Args) -&gt; a,\n) -&gt; Definition(a, b, c)\n Replace the widget that this `Definition` uses for rendering the field.  Most\n HTML inputs can be interchangeable, they all generate a `String` after all,\n but not all are the best UX.  This allows you to choose the one that is the\n most appropriate for your field.\nSynonyms:\nformz/definition.set_widget\nformz/definition set_widget","url":"formz/definition.html#set_widget"},{"doc":"formz/definition","title":"validate","content":"pub fn validate(\n  def: Definition(a, b, c),\n  fun: fn(b) -&gt; Result(b, String),\n) -&gt; Definition(a, b, c)\n Chain additional validation onto the `parse` function.  This is\n useful if you don&#39;t need to change the returned type, but might have\n additional constraints.  Like say, requiring a `String` to be at least\n a certain length, or that an Int must be positive.\n\n ### Example\n \n field\n   |&gt; validate(fn(i) {\n     case i &gt; 0 {\n       True -&gt; Ok(i)\n       False -&gt; Error(&quot;must be positive&quot;)\n     }\n   }),\n \nSynonyms:\nformz/definition.validate\nformz/definition validate","url":"formz/definition.html#validate"},{"doc":"formz/definition","title":"formz/definition","content":" A `Definition` is the second argument needed to add a field to a form. It\n is what describes how a field works, e.g. how it looks and how it&#39;s parsed.\n It is the heavy compared to the lightness of a [Field](https://hexdocs.pm/formz/formz/field.html);\n they take a bit more work to make as they are intended to be reusable.\n\n The first role of a `Defintion` is to generate the HTML widget for the field.\n This library is format-agnostic and you can generate HTML widgets as raw\n strings, Lustre elements, Nakai nodes, something else, etc. There are\n currently three `formz` libraries that provide common field definitions for the\n most common HTML formats.\n\n - [formz_string](https://hexdocs.pm/formz_string/)\n - [formz_nakai](https://hexdocs.pm/formz_nakai/)\n - [formz_lustre](https://hexdocs.pm/formz_lustre/) (untested in a browser,\n   would it be useful there??)\n\n The second role  of a `Definition` is to parse the data from the field.\n There are a two parts\n to this, as how you parse a field&#39;s value depends on if it is optional or\n required.  Not all scenarios can be cookie-cutter placed into an `Option`.\n So you need to provide two parse functions, one for when\n a field is required, and a second for when it&#39;s optional.\n\n ### Example password field definition\n\n ```gleam\n /// you won&#39;t often need to do this directly (I think??).  The idea is that\n /// there&#39;d be libs with the definitions you need.\n\n import formz/definition.{Definition}\n import formz/field\n import formz/validation\n import formz/widget\n import lustre/attribute\n import lustre/element\n import lustre/element/html\n\n fn password_widget(\n   field: field.Field,\n   args: widget.Args,\n ) -&gt; element.Element(msg) {\n   html.input([\n     attribute.type_(&quot;password&quot;),\n     attribute.name(field.name),\n     attribute.id(args.id),\n     attribute.attribute(&quot;aria-labelledby&quot;, field.label),\n   ])\n }\n\n pub fn password_field() {\n   Definition(\n     widget: password_widget,\n     parse: validation.string,\n     optional_parse: fn(parse, str) {\n       case str {\n         &quot;&quot; -&gt; Ok(option.None)\n         _ -&gt; parse(str)\n       }\n     },\n     // We need to have a stub value for each parser. The stubs are used when\n     // building the decoder and parse functions for the form.\n     stub: &quot;&quot;,\n     optional_stub: option.None,\n   )\n }\n ```","url":"formz/definition.html"},{"doc":"formz/formz_builder","title":"Form","content":"pub opaque type Form(format, output, decoder, has_decoder)\n\n\nSynonyms:\nformz/formz_builder.Form\nformz/formz_builder Form","url":"formz/formz_builder.html#Form"},{"doc":"formz/formz_builder","title":"HasDecoder","content":"pub type HasDecoder\n\n\nSynonyms:\nformz/formz_builder.HasDecoder\nformz/formz_builder HasDecoder","url":"formz/formz_builder.html#HasDecoder"},{"doc":"formz/formz_builder","title":"NoDecoder","content":"pub type NoDecoder\n\n\nSynonyms:\nformz/formz_builder.NoDecoder\nformz/formz_builder NoDecoder","url":"formz/formz_builder.html#NoDecoder"},{"doc":"formz/formz_builder","title":"data","content":"pub fn data(\n  form: Form(a, b, c, d),\n  input_data: List(#(String, String)),\n) -&gt; Form(a, b, c, d)\n\nSynonyms:\nformz/formz_builder.data\nformz/formz_builder data","url":"formz/formz_builder.html#data"},{"doc":"formz/formz_builder","title":"decodes","content":"pub fn decodes(\n  form: Form(a, b, c, d),\n  decoder: c,\n) -&gt; Form(a, b, c, HasDecoder)\n\nSynonyms:\nformz/formz_builder.decodes\nformz/formz_builder decodes","url":"formz/formz_builder.html#decodes"},{"doc":"formz/formz_builder","title":"get","content":"pub fn get(\n  form: Form(a, b, c, d),\n  name: String,\n) -&gt; Result(FormItem(a), Nil)\n\nSynonyms:\nformz/formz_builder.get\nformz/formz_builder get","url":"formz/formz_builder.html#get"},{"doc":"formz/formz_builder","title":"get_fields","content":"pub fn get_fields(form: Form(a, b, c, d)) -&gt; List(Field)\n\nSynonyms:\nformz/formz_builder.get_fields\nformz/formz_builder get_fields","url":"formz/formz_builder.html#get_fields"},{"doc":"formz/formz_builder","title":"items","content":"pub fn items(form: Form(a, b, c, d)) -&gt; List(FormItem(a))\n\nSynonyms:\nformz/formz_builder.items\nformz/formz_builder items","url":"formz/formz_builder.html#items"},{"doc":"formz/formz_builder","title":"new","content":"pub fn new() -&gt; Form(a, b, b, NoDecoder)\n\nSynonyms:\nformz/formz_builder.new\nformz/formz_builder new","url":"formz/formz_builder.html#new"},{"doc":"formz/formz_builder","title":"optional","content":"pub fn optional(\n  previous_form: Form(a, fn(b) -&gt; c, d, e),\n  field: Field,\n  definition: Definition(a, f, b),\n) -&gt; Form(a, c, d, e)\n\nSynonyms:\nformz/formz_builder.optional\nformz/formz_builder optional","url":"formz/formz_builder.html#optional"},{"doc":"formz/formz_builder","title":"parse","content":"pub fn parse(\n  form: Form(a, b, c, HasDecoder),\n) -&gt; Result(b, Form(a, b, c, HasDecoder))\n\nSynonyms:\nformz/formz_builder.parse\nformz/formz_builder parse","url":"formz/formz_builder.html#parse"},{"doc":"formz/formz_builder","title":"parse_then_try","content":"pub fn parse_then_try(\n  form: Form(a, b, c, HasDecoder),\n  apply fun: fn(Form(a, b, c, HasDecoder), b) -&gt;\n    Result(d, Form(a, b, c, HasDecoder)),\n) -&gt; Result(d, Form(a, b, c, HasDecoder))\n\nSynonyms:\nformz/formz_builder.parse_then_try\nformz/formz_builder parse_then_try","url":"formz/formz_builder.html#parse_then_try"},{"doc":"formz/formz_builder","title":"require","content":"pub fn require(\n  previous_form: Form(a, fn(b) -&gt; c, d, e),\n  field: Field,\n  definition: Definition(a, b, f),\n) -&gt; Form(a, c, d, e)\n\nSynonyms:\nformz/formz_builder.require\nformz/formz_builder require","url":"formz/formz_builder.html#require"},{"doc":"formz/formz_builder","title":"subform","content":"pub fn subform(\n  previous_form: Form(a, fn(b) -&gt; c, d, e),\n  details: SubForm,\n  sub: Form(a, b, f, HasDecoder),\n) -&gt; Form(a, c, d, e)\n\nSynonyms:\nformz/formz_builder.subform\nformz/formz_builder subform","url":"formz/formz_builder.html#subform"},{"doc":"formz/formz_builder","title":"update","content":"pub fn update(\n  form: Form(a, b, c, d),\n  name: String,\n  fun: fn(FormItem(a)) -&gt; FormItem(a),\n) -&gt; Form(a, b, c, e)\n\nSynonyms:\nformz/formz_builder.update\nformz/formz_builder update","url":"formz/formz_builder.html#update"},{"doc":"formz/formz_builder","title":"update_field","content":"pub fn update_field(\n  form: Form(a, b, c, d),\n  name: String,\n  fun: fn(Field) -&gt; Field,\n) -&gt; Form(a, b, c, d)\n\nSynonyms:\nformz/formz_builder.update_field\nformz/formz_builder update_field","url":"formz/formz_builder.html#update_field"},{"doc":"formz/formz_builder","title":"update_subform","content":"pub fn update_subform(\n  form: Form(a, b, c, d),\n  name: String,\n  fun: fn(SubForm) -&gt; SubForm,\n) -&gt; Form(a, b, c, d)\n\nSynonyms:\nformz/formz_builder.update_subform\nformz/formz_builder update_subform","url":"formz/formz_builder.html#update_subform"},{"doc":"formz/formz_builder","title":"validate","content":"pub fn validate(\n  form: Form(a, b, c, HasDecoder),\n  names: List(String),\n) -&gt; Form(a, b, c, HasDecoder)\n\nSynonyms:\nformz/formz_builder.validate\nformz/formz_builder validate","url":"formz/formz_builder.html#validate"},{"doc":"formz/formz_builder","title":"validate_all","content":"pub fn validate_all(\n  form: Form(a, b, c, HasDecoder),\n) -&gt; Form(a, b, c, HasDecoder)\n\nSynonyms:\nformz/formz_builder.validate_all\nformz/formz_builder validate_all","url":"formz/formz_builder.html#validate_all"},{"doc":"formz/formz_builder","title":"formz/formz_builder","content":"","url":"formz/formz_builder.html"},{"doc":"formz/formz_use","title":"Form","content":"pub opaque type Form(format, output)\n\n\nSynonyms:\nformz/formz_use.Form\nformz/formz_use Form","url":"formz/formz_use.html#Form"},{"doc":"formz/formz_use","title":"create_form","content":"pub fn create_form(thing: a) -&gt; Form(b, a)\n\nSynonyms:\nformz/formz_use.create_form\nformz/formz_use create_form","url":"formz/formz_use.html#create_form"},{"doc":"formz/formz_use","title":"data","content":"pub fn data(\n  form: Form(a, b),\n  input_data: List(#(String, String)),\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.data\nformz/formz_use data","url":"formz/formz_use.html#data"},{"doc":"formz/formz_use","title":"get","content":"pub fn get(\n  form: Form(a, b),\n  name: String,\n) -&gt; Result(FormItem(a), Nil)\n\nSynonyms:\nformz/formz_use.get\nformz/formz_use get","url":"formz/formz_use.html#get"},{"doc":"formz/formz_use","title":"get_fields","content":"pub fn get_fields(form: Form(a, b)) -&gt; List(Field)\n\nSynonyms:\nformz/formz_use.get_fields\nformz/formz_use get_fields","url":"formz/formz_use.html#get_fields"},{"doc":"formz/formz_use","title":"items","content":"pub fn items(form: Form(a, b)) -&gt; List(FormItem(a))\n\nSynonyms:\nformz/formz_use.items\nformz/formz_use items","url":"formz/formz_use.html#items"},{"doc":"formz/formz_use","title":"optional","content":"pub fn optional(\n  field: Field,\n  is definition: Definition(a, b, c),\n  rest fun: fn(c) -&gt; Form(a, d),\n) -&gt; Form(a, d)\n\nSynonyms:\nformz/formz_use.optional\nformz/formz_use optional","url":"formz/formz_use.html#optional"},{"doc":"formz/formz_use","title":"parse","content":"pub fn parse(form: Form(a, b)) -&gt; Result(b, Form(a, b))\n\nSynonyms:\nformz/formz_use.parse\nformz/formz_use parse","url":"formz/formz_use.html#parse"},{"doc":"formz/formz_use","title":"parse_then_try","content":"pub fn parse_then_try(\n  form: Form(a, b),\n  apply fun: fn(Form(a, b), b) -&gt; Result(c, Form(a, b)),\n) -&gt; Result(c, Form(a, b))\n\nSynonyms:\nformz/formz_use.parse_then_try\nformz/formz_use parse_then_try","url":"formz/formz_use.html#parse_then_try"},{"doc":"formz/formz_use","title":"require","content":"pub fn require(\n  field: Field,\n  is definition: Definition(a, b, c),\n  rest fun: fn(b) -&gt; Form(a, d),\n) -&gt; Form(a, d)\n\nSynonyms:\nformz/formz_use.require\nformz/formz_use require","url":"formz/formz_use.html#require"},{"doc":"formz/formz_use","title":"subform","content":"pub fn subform(\n  details: SubForm,\n  sub: Form(a, b),\n  fun: fn(b) -&gt; Form(a, c),\n) -&gt; Form(a, c)\n\nSynonyms:\nformz/formz_use.subform\nformz/formz_use subform","url":"formz/formz_use.html#subform"},{"doc":"formz/formz_use","title":"update","content":"pub fn update(\n  form: Form(a, b),\n  name: String,\n  fun: fn(FormItem(a)) -&gt; FormItem(a),\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.update\nformz/formz_use update","url":"formz/formz_use.html#update"},{"doc":"formz/formz_use","title":"update_field","content":"pub fn update_field(\n  form: Form(a, b),\n  name: String,\n  fun: fn(Field) -&gt; Field,\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.update_field\nformz/formz_use update_field","url":"formz/formz_use.html#update_field"},{"doc":"formz/formz_use","title":"update_subform","content":"pub fn update_subform(\n  form: Form(a, b),\n  name: String,\n  fun: fn(SubForm) -&gt; SubForm,\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.update_subform\nformz/formz_use update_subform","url":"formz/formz_use.html#update_subform"},{"doc":"formz/formz_use","title":"validate","content":"pub fn validate(\n  form: Form(a, b),\n  names: List(String),\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.validate\nformz/formz_use validate","url":"formz/formz_use.html#validate"},{"doc":"formz/formz_use","title":"validate_all","content":"pub fn validate_all(form: Form(a, b)) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.validate_all\nformz/formz_use validate_all","url":"formz/formz_use.html#validate_all"},{"doc":"formz/formz_use","title":"formz/formz_use","content":"","url":"formz/formz_use.html"},{"doc":"formz/validation","title":"and","content":"pub fn and(\n  previous: fn(a) -&gt; Result(b, String),\n  next: fn(b) -&gt; Result(c, String),\n) -&gt; fn(a) -&gt; Result(c, String)\n Chain validations together.\n\n ## Examples\n\n \n let is_even = fn(num) {\n   case num % 2 == 0 {\n     True -&gt; Ok(num)\n     False -&gt; Error(&quot;must be even&quot;)\n   }\n }\n let check = and(int, is_even)\n\n check(&quot;2&quot;)\n # -&gt; Ok(2)\n\n check(&quot;hi&quot;)\n # -&gt; Error(&quot;must be a whole number&quot;)\n\n check(&quot;1&quot;)\n # -&gt; Error(&quot;must be even&quot;)\nSynonyms:\nformz/validation.and\nformz/validation and","url":"formz/validation.html#and"},{"doc":"formz/validation","title":"email","content":"pub fn email(input: String) -&gt; Result(String, String)\n Parse the input as a String that looks like an email address, i.e. it\n contains an `@` character, with at least one other character on either\n side\n\n (this behavior more closely matches what the browser does than just\n checking for an `@`).\n\n ## Examples\n\n \n email(&quot;hello@example.com&quot;)\n # -&gt; Ok(&quot;hello@example.com&quot;)\n \n \n email(&quot;@&quot;)\n # -&gt; Error(&quot;Must be an email address&quot;)\n \n \n email(&quot;hello&quot;)\n # -&gt; Error(&quot;Must be an email address&quot;)\n \nSynonyms:\nformz/validation.email\nformz/validation email","url":"formz/validation.html#email"},{"doc":"formz/validation","title":"int","content":"pub fn int(str: String) -&gt; Result(Int, String)\n Parse the input as an int.\n\n ## Examples\n\n \n int(&quot;1&quot;)\n # -&gt; Ok(1)\n \n\n \n int(&quot;3.4&quot;)\n # -&gt; Error(&quot;Must be a whole number&quot;)\n \n \n int(&quot;hello&quot;)\n # -&gt; Error(&quot;Must be a whole number&quot;)\n \nSynonyms:\nformz/validation.int\nformz/validation int","url":"formz/validation.html#int"},{"doc":"formz/validation","title":"list_item_by_index","content":"pub fn list_item_by_index(\n  variants: List(a),\n) -&gt; fn(String) -&gt; Result(a, String)\n Validates that the input is one from a list of allowed values. Takes a list\n of Gleam values that can be chosen.  This uses the index of the item in to\n find the desired value.\n\n ## Examples\n\n \n enum([&quot;One&quot;,&quot;Two&quot;,&quot;Three&quot;])(&quot;1&quot;)\n # -&gt; Ok(&quot;Two&quot;)\n \n\n \n enum([True, False])(&quot;42&quot;)\n # -&gt; Error(&quot;must be an item in list&quot;)\n \n\n \n enum([True, False])(&quot;ok&quot;)\n # -&gt; Error(&quot;must be an item in list&quot;)\n \nSynonyms:\nformz/validation.list_item_by_index\nformz/validation list_item_by_index","url":"formz/validation.html#list_item_by_index"},{"doc":"formz/validation","title":"non_empty_string","content":"pub fn non_empty_string(str: String) -&gt; Result(String, String)\n\nSynonyms:\nformz/validation.non_empty_string\nformz/validation non_empty_string","url":"formz/validation.html#non_empty_string"},{"doc":"formz/validation","title":"number","content":"pub fn number(str: String) -&gt; Result(Float, String)\n Parse the input as a float.  this is forgiving and will also parse\n ints into floats.\n\n ## Examples\n\n \n number(&quot;1&quot;)\n # -&gt; Ok(1.0)\n \n\n \n number(&quot;3.4&quot;)\n # -&gt; Ok(3.4)\n \n \n number(&quot;hello&quot;)\n # -&gt; Error(&quot;Must be a number&quot;)\n \nSynonyms:\nformz/validation.number\nformz/validation number","url":"formz/validation.html#number"},{"doc":"formz/validation","title":"on","content":"pub fn on(val: String) -&gt; Result(Bool, String)\n Parse the input as a boolean, where only &quot;on&quot; is True and allowed.\n All other values are an error.  This is useful for HTML checkboxes, which\n the browser sends the empty string if unchecked, and `&quot;on&quot;` if  checked.\n\n ## Examples\n\n \n on(&quot;on&quot;)\n # -&gt; Ok(True)\n \n\n \n on(&quot;&quot;)\n # -&gt; Error(&quot;is required&quot;)\n \n\n \n on(&quot;hi&quot;)\n # -&gt; Error(&quot;is required&quot;)\n \nSynonyms:\nformz/validation.on\nformz/validation on","url":"formz/validation.html#on"},{"doc":"formz/validation","title":"replace_error","content":"pub fn replace_error(\n  previous: fn(a) -&gt; Result(b, String),\n  error: String,\n) -&gt; fn(a) -&gt; Result(b, String)\n Replace the error message of a validation with a new one.  Most of the built-in\n error messages are pretty rudimentary.\nSynonyms:\nformz/validation.replace_error\nformz/validation replace_error","url":"formz/validation.html#replace_error"},{"doc":"formz/validation","title":"formz/validation","content":"","url":"formz/validation.html"}]);